
-- n-Bit Shift Register, Slide 125.
-- Tobias Schubert, 2011.

library ieee;
use ieee.std_logic_1164.all;

entity shift_register_nbit is
	generic ( n: positive := 32 );
	port ( din, clk, set, reset: in std_logic; dout: out std_logic );
end shift_register_nbit;

architecture structural of shift_register_nbit is
	component flipflop
		port ( data, clk, set, reset: in std_logic; q: out std_logic );
	end component;
	signal T: std_logic_vector (n-2 downto 0);
begin
	label1: for i in n-1 downto 0 generate
		label2: if i = n-1 generate
			bitL: flipflop port map ( din, clk, set, reset, T(n-2) ); end generate;
		label3: if i > 0 and i < n-1 generate
			bitM: flipflop port map ( T(i), clk, set, reset, T(i-1) ); end generate;
		label4: if i = 0 generate
			bitR: flipflop port map ( T(0), clk, set, reset, dout ); end generate;
	end generate;
end structural; 

library ieee;
use ieee.std_logic_1164.all;

entity flipflop is
	port ( data, clk, set, reset: in std_logic; q: out std_logic );
end flipflop;

architecture behavior of flipflop is
begin
	process ( clk, set, reset )
	begin
		if reset = '1' then
			q <= '0';
		elsif set = '1' then
			q <= '1';
		elsif (clk'event and clk = '1') then
			q <= data;
		end if;
	end process;
end behavior;